@@ -1,3 +1,7 @@
+"""A balance highlighter.
+
+For: Achaea.
+"""
 from mudpyl.colours import HexFGCode
 from mudpyl.triggers import non_binding_trigger
 
