@@ -1,5 +1,6 @@
 """Utility for passing around lines with their colour information."""
 from bisect import insort
+from itertools import izip, tee, chain
 
 def iadjust(ind, start, adj):
     """Moves the ind along by adj amount, unless ind is before start, when it
@@ -15,14 +16,19 @@ def iadjust(ind, start, adj):
         return start
     return ind + adj
 
-def _pairwise(seq):
+def pairwise(seq):
     """Return a list pairwise.
 
     Example::
         >>> pairwise([1, 2, 3, 4])
         [(1, 2), (2, 3), (3, 4)]
     """
-    return zip(seq, seq[1:])
+    a, b = tee(seq)
+    try:
+        b.next()
+    except StopIteration:
+        pass
+    return izip(a, b)
 
 class _LoopingLast(list):
     """A list whose final value is repeated infinitely."""
@@ -42,14 +48,16 @@ class RunLengthList(object):
     Bad Things (related to insort, order and consistency) may happen.
     """
 
-    def __init__(self, values):
-        #make sure we don't choke at a later date on a generator
-        self.values = list(values)
+    def __init__(self, values, _normalised = False):
+        #pairwise works off of itertools.tee, so this isn't broken for
+        #generators.
+        self.values = values
+        if not _normalised:
+            self._normalise()
         if not self.values:
             raise ValueError("The list of values must not be empty.")
         if self.values[0][0] != 0:
             raise ValueError("All the indices must be specified - no gap!")
-        self._normalise()
 
     def as_populated_list(self):
         """Return a normal array of values. 
@@ -58,7 +66,7 @@ class RunLengthList(object):
         loops infinitely.
         """
         res = []
-        for (start, val), (end, _) in _pairwise(self.values):
+        for (start, val), (end, _) in pairwise(self.values):
             res += [val] * (end - start)
         #our very last value won't be plucked by the above loop, so we add
         #it here.
@@ -68,22 +76,23 @@ class RunLengthList(object):
 
     def as_pruned_index_list(self):
         """Return an unwrapped unredundant run-length list."""
+        #don't want to share references of mutable data
+        return self.values[:]
+
+    def _normalise(self):
+        """Remove redundancies."""
         res = []
         cur = object() #sentinel
         #we can have None here, because only the first member of the pairwise
         #(index, value) will be used, and it'll only be right at the end. In
         #fact, only the first one being used is the very reason we need to
         #do this: how else will the final value get out there?
-        values = self.values + [(object(), None)]
-        for (start, val), (end, _) in _pairwise(values):
+        values = chain(self.values, [(object(), None)])
+        for (start, val), (end, _) in pairwise(values):
             if start != end and cur != val:
                 cur = val
                 res.append((start, val))
-        return res
-
-    def _normalise(self):
-        """Remove redundancies."""
-        self.values = self.as_pruned_index_list()
+        self.values = res
 
     def add_colour(self, ind, value):
         """Add a value starting at a specific point."""
@@ -113,7 +122,10 @@ class RunLengthList(object):
         for num in range(len(self.values)):
             ind, col = self.values[num]
             self.values[num] = (iadjust(ind, start, change), col)
-        self._normalise()
+        if change < 0:
+            #if the change moves things forwards, things may be made redundant
+            #thus we must normalise for this case
+            self._normalise()
 
     def blank_between(self, start, end):
         """Delete a span of values between given indexes."""
@@ -148,7 +160,7 @@ class RunLengthList(object):
 
     def copy(self):
         """Return a deep copy of ourselves."""
-        return RunLengthList(self.as_pruned_index_list())
+        return RunLengthList(self.as_pruned_index_list(), _normalised = True)
 
 class Metaline(object):
     """A line plus some metadata.
