@@ -2,8 +2,7 @@
 to logs).
 """
 from textwrap import TextWrapper
-from mudpyl.colours import WHITE, BLACK, HexFGCode, HexBGCode, fg_code, \
-                           bg_code
+from mudpyl.colours import WHITE, BLACK, fg_code, bg_code
 
 class OutputManager(object):
     """Handles:
@@ -15,7 +14,9 @@ class OutputManager(object):
     def __init__(self, factory):
         self.factory = factory
         self.outputs = []
+        self.metaline_peekers = []
         self.last_line_end = None
+        #sensible defaults
         self.fore = fg_code(WHITE, False)
         self.back = bg_code(BLACK)
         self.wrapper = TextWrapper(width = 100, 
@@ -25,6 +26,10 @@ class OutputManager(object):
         """Add another output that wants to receive stuff."""
         self.outputs.append(output)
 
+    def add_metaline_peeker(self, peeker):
+        """Add an object that wants the line-wrapped metalines."""
+        self.metaline_peekers.append(peeker)
+
     def _wrap_line(self, metaline):
         """Break an incoming line into sensible lengths."""
         #this needs to be before the futzing with NLs and GA, because textwrap
@@ -54,8 +59,11 @@ class OutputManager(object):
             if self.last_line_end == 'hard' or not metaline.soft_line_start:
                 metaline.insert(0, '\n')
 
-        allcolours = list(sorted(metaline.fores.as_pruned_index_list() + 
-                                 metaline.backs.as_pruned_index_list()))
+        for peeker in self.metaline_peekers:
+            peeker.peek_metaline(metaline)
+
+        allcolours = sorted(metaline.fores.as_pruned_index_list() + 
+                            metaline.backs.as_pruned_index_list())
 
         self._actually_write_to_screen(allcolours, metaline.line)
 
@@ -93,11 +101,11 @@ class OutputManager(object):
     def _change_colour(self, change):
         """Change the colour that new text will be written as.
         """
-        if isinstance(change, HexBGCode):
+        if change.ground == 'back':
             self.back = change
             for output in self.outputs:
                 output.bg_changed(change)
-        elif isinstance(change, HexFGCode):
+        elif change.ground == 'fore':
             self.fore = change
             for output in self.outputs:
                 output.fg_changed(change)
